#Author: David Szybala (www.davidszybala.com)
#Title: removeUnusedUVs.py
#Date: 10/23/14
#Description: This script will remove all UV sets that are not being used (i.e. linked to a texture).
#Installation: Place this file in your maya scripts directory (e.g. ..\My Documents\maya\scripts) then run the following python commands:
#   import removeUnusedUVs
#   removeUnusedUVs.main()
import maya.cmds as cmds
import maya.mel as mel


def main():
    currentSelection = cmds.ls(selection=True, long=True, noIntermediate=True)
    currentChildren = []
    allMeshItems = []
    allUvSets = []
    uvSetLinks = []
    deleteUvSet = True
    counter = 0
    
    if len(currentSelection) > 0:
        for currentObj in currentSelection:
            if cmds.nodeType(currentObj) == "mesh":
                allMeshItems.append(currentObj)
            else:
                if cmds.nodeType(currentObj) == "transform":
                    currentChildren = cmds.listRelatives(currentObj, shapes=True, noIntermediate=True, path=True)
                    if currentChildren != None:
                        for currentChild in currentChildren:
                            if cmds.nodeType(currentChild) == "mesh":
                                allMeshItems.append(currentChild)
    else:
        allMeshItems = cmds.ls(type="mesh", long=True, noIntermediate=True)
    if len(allMeshItems) > 0:
        if (cmds.window("polyTexturePlacementPanel1Window", exists=True)):
            cmds.deleteUI("polyTexturePlacementPanel1Window")
        cmds.select(cl=True)
        gMainProgressBar = mel.eval('$tmp = $gMainProgressBar');
        cmds.progressBar(gMainProgressBar, edit=True, beginProgress=True, isInterruptable=False, status="Removing unused UVs. Please wait...", maxValue=(len(allMeshItems)))
        for currentObj in allMeshItems:
            allUvSets = cmds.polyUVSet(currentObj, query=True, allUVSets=True)
            if allUvSets != None:
                for i in range((len(allUvSets) - 1), -1, -1):
                    uvSetLinks = cmds.uvLink(query=True, uvSet=(currentObj + ".uvSet[" + str(i) + "].uvSetName"))
                    if uvSetLinks == []:
                        if (cmds.polyEvaluate(currentObj, uvcoord=True, uvSetName=(allUvSets[i]))) > 0:
                            if allUvSets[i] == "map1" or i == 0:
                                deleteUvSet = False
                            else:
                                deleteUvSet = True
                            cmds.polyUVSet(currentObj, currentUVSet=True, uvSet=(allUvSets[i]))
                            cmds.polyMapDel(currentObj, ch=False)
                            counter += 1
                            if deleteUvSet:
                                cmds.polyUVSet(currentObj, delete=True, uvSet=(allUvSets[i]))
            cmds.select(cl=True)
            cmds.progressBar(gMainProgressBar, edit=True, step=1)
        cmds.progressBar(gMainProgressBar, edit=True, endProgress=True)
        print("Unused UVs have been removed. Total: " + str(counter) + ".\n"),
    else:
        print("ERROR: No mesh items found.\n"),


if __name__ == "__main__":
    main()